@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -54,7 +55,8 @@ Plugin_table table_setup_instruments::m_table_def(
     "  ENABLED ENUM ('YES', 'NO') not null,\n"
     "  TIMED ENUM ('YES', 'NO'),\n"
     "  PROPERTIES SET('singleton', 'progress', 'user', 'global_statistics', "
-    "'mutable') not null,\n"
+    "'mutable', 'controlled_by_default') not null,\n"
+    "  FLAGS SET('controlled'),\n"
     "  VOLATILITY int not null,\n"
     "  DOCUMENTATION LONGTEXT,\n"
     "  PRIMARY KEY (NAME) USING HASH\n",
@@ -97,7 +99,7 @@ PFS_engine_table *table_setup_instruments::create(PFS_engine_table_share *) {
   return new table_setup_instruments();
 }
 
-ha_rows table_setup_instruments::get_row_count(void) {
+ha_rows table_setup_instruments::get_row_count() {
   return wait_class_max + stage_class_max + statement_class_max +
          transaction_class_max + memory_class_max + error_class_max;
 }
@@ -105,12 +107,12 @@ ha_rows table_setup_instruments::get_row_count(void) {
 table_setup_instruments::table_setup_instruments()
     : PFS_engine_table(&m_share, &m_pos), m_pos(), m_next_pos() {}
 
-void table_setup_instruments::reset_position(void) {
+void table_setup_instruments::reset_position() {
   m_pos.reset();
   m_next_pos.reset();
 }
 
-int table_setup_instruments::rnd_next(void) {
+int table_setup_instruments::rnd_next() {
   PFS_instr_class *instr_class = nullptr;
   PFS_builtin_memory_class *pfs_builtin;
   bool update_enabled;
@@ -255,7 +257,7 @@ int table_setup_instruments::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_setup_instruments::index_init(uint idx MY_ATTRIBUTE((unused)), bool) {
+int table_setup_instruments::index_init(uint idx [[maybe_unused]], bool) {
   PFS_index_setup_instruments *result;
 
   assert(idx == 0);
@@ -266,7 +268,7 @@ int table_setup_instruments::index_init(uint idx MY_ATTRIBUTE((unused)), bool) {
   return 0;
 }
 
-int table_setup_instruments::index_next(void) {
+int table_setup_instruments::index_next() {
   PFS_instr_class *instr_class = nullptr;
   PFS_builtin_memory_class *pfs_builtin;
   bool update_enabled;
@@ -355,6 +357,7 @@ int table_setup_instruments::make_row(PFS_instr_class *klass,
   m_row.m_instr_class = klass;
   m_row.m_update_enabled = update_enabled;
   m_row.m_update_timed = klass->can_be_timed();
+  m_row.m_update_flags = klass->can_be_enforced();
 
   return 0;
 }
@@ -364,6 +367,7 @@ int table_setup_instruments::read_row_values(TABLE *table, unsigned char *buf,
   Field *f;
   const char *doc;
   uint properties;
+  uint enforced;
 
   /* Set the null bits */
   assert(table->s->null_bytes == 1);
@@ -378,8 +382,8 @@ int table_setup_instruments::read_row_values(TABLE *table, unsigned char *buf,
     if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
       switch (f->field_index()) {
         case 0: /* NAME */
-          set_field_varchar_utf8(f, m_row.m_instr_class->m_name,
-                                 m_row.m_instr_class->m_name_length);
+          set_field_varchar_utf8mb4(f, m_row.m_instr_class->m_name.str(),
+                                    m_row.m_instr_class->m_name.length());
           break;
         case 1: /* ENABLED */
           set_field_enum(f,
@@ -410,12 +414,26 @@ int table_setup_instruments::read_row_values(TABLE *table, unsigned char *buf,
           if (m_row.m_instr_class->is_global()) {
             properties |= INSTR_PROPERTIES_SET_GLOBAL_STAT;
           }
+          if (m_row.m_instr_class->has_default_memory_cnt()) {
+            properties |= INSTR_PROPERTIES_SET_QUOTA_BY_DEFAULT;
+          }
           set_field_set(f, properties);
           break;
-        case 4: /* VOLATILITY */
+        case 4: /* FLAGS */
+          if (m_row.m_update_flags) {
+            enforced = 0;
+            if (m_row.m_instr_class->has_enforced_memory_cnt()) {
+              enforced |= INSTR_FLAGS_SET_CONTROLLED;
+            }
+            set_field_set(f, enforced);
+          } else {
+            f->set_null();
+          }
+          break;
+        case 5: /* VOLATILITY */
           set_field_ulong(f, m_row.m_instr_class->m_volatility);
           break;
-        case 5: /* DOCUMENTATION */
+        case 6: /* DOCUMENTATION */
           doc = m_row.m_instr_class->m_documentation;
           if (doc != nullptr) {
             set_field_blob(f, doc, strlen(doc));
@@ -442,29 +460,44 @@ int table_setup_instruments::update_row_values(TABLE *table,
   for (; (f = *fields); fields++) {
     if (bitmap_is_set(table->write_set, f->field_index())) {
       switch (f->field_index()) {
-        case 0: /* NAME */
-          return HA_ERR_WRONG_COMMAND;
         case 1: /* ENABLED */
           /* Do not raise error if m_update_enabled is false, silently ignore.
            */
           if (m_row.m_update_enabled) {
             value = (enum_yes_no)get_field_enum(f);
-            m_row.m_instr_class->m_enabled = (value == ENUM_YES) ? true : false;
+            m_row.m_instr_class->m_enabled = (value == ENUM_YES);
           }
           break;
         case 2: /* TIMED */
           /* Do not raise error if m_update_timed is false, silently ignore. */
           if (m_row.m_update_timed) {
             value = (enum_yes_no)get_field_enum(f);
-            m_row.m_instr_class->m_timed = (value == ENUM_YES) ? true : false;
+            m_row.m_instr_class->m_timed = (value == ENUM_YES);
+          }
+          break;
+        case 4: /* FLAGS */
+          /*
+            Silently ignore the update if:
+            - the instrument is not a memory instrument (m_update_flags
+              is false)
+            - the instrument is global (controlled memory depends on
+              per thread statistics)
+           */
+          if (m_row.m_update_flags && !m_row.m_instr_class->is_global()) {
+            ulonglong enforced_value = 0;
+            uint enforced_flags = 0;
+            /* Treat FLAGS = NULL as FLAGS = '' */
+            if (!f->is_null()) {
+              enforced_value = get_field_set(f);
+            }
+            if (enforced_value & INSTR_FLAGS_SET_CONTROLLED) {
+              enforced_flags |= PSI_FLAG_MEM_COLLECT;
+            }
+            m_row.m_instr_class->set_enforced_flags(enforced_flags);
           }
           break;
-        case 3: /* PROPERTIES */
-        case 4: /* VOLATILITY */
-        case 5: /* DOCUMENTATION */
-          return HA_ERR_WRONG_COMMAND;
         default:
-          assert(false);
+          return HA_ERR_WRONG_COMMAND;
       }
     }
   }
